@@ -14,7 +14,34 @@ def constant(s: str):
     return Enum(value=s, names={s: s}, type=str)
 
 
-class OperationAnnotate(BaseModel):
+class RunnerAllowList(BaseModel):
+    allowlist: Optional[HashableSequence[str]]
+    _nonzero_allowlist = validator("allowlist", allow_reuse=True)(nonzero_validator)
+
+    class Config:
+        extra = "forbid"
+
+
+class RunnerDenyList(BaseModel):
+    denylist: Optional[HashableSequence[str]]
+    _nonzero_denylist = validator("denylist", allow_reuse=True)(nonzero_validator)
+
+    class Config:
+        extra = "forbid"
+
+
+class RunnerParameters(BaseModel):
+    __root__: Optional[Union[RunnerAllowList, RunnerDenyList]]
+
+
+class BaseOperation(BaseModel):
+    runner_parameters: Optional[RunnerParameters]
+
+    class Config:
+        extra = "forbid"
+
+
+class OperationAnnotate(BaseOperation):
     id: constant("annotate")
     parameters: Optional[Any]
 
@@ -26,7 +53,7 @@ class AnnotateEdgesParameters(BaseModel):
     attributes: Optional[HashableSequence[str]]
 
 
-class OperationAnnotateEdges(BaseModel):
+class OperationAnnotateEdges(BaseOperation):
     id: constant("annotate_edges")
     parameters: Optional[AnnotateEdgesParameters]
 
@@ -38,7 +65,7 @@ class AnnotateNodesParameters(BaseModel):
     attributes: Optional[HashableSequence[str]]
 
 
-class OperationAnnotateNodes(BaseModel):
+class OperationAnnotateNodes(BaseOperation):
     id: constant("annotate_nodes")
     parameters: Optional[AnnotateNodesParameters]
 
@@ -46,7 +73,7 @@ class OperationAnnotateNodes(BaseModel):
         extra = "forbid"
 
 
-class OperationBind(BaseModel):
+class OperationBind(BaseOperation):
     id: constant("bind")
     parameters: Optional[Any]
 
@@ -54,7 +81,7 @@ class OperationBind(BaseModel):
         extra = "forbid"
 
 
-class OperationCompleteResults(BaseModel):
+class OperationCompleteResults(BaseOperation):
     id: constant("complete_results")
     parameters: Optional[Any]
 
@@ -67,7 +94,7 @@ class EnrichResultsParameters(BaseModel):
     qnode_keys: Optional[HashableSequence[str]]
 
 
-class OperationEnrichResults(BaseModel):
+class OperationEnrichResults(BaseOperation):
     id: constant("enrich_results")
     parameters: Optional[EnrichResultsParameters]
 
@@ -92,10 +119,10 @@ class FillDenyParameters(BaseModel):
 
 
 class FillParameters(BaseModel):
-    _root_: Union[FillAllowParameters, FillDenyParameters]
+    __root__: Union[FillAllowParameters, FillDenyParameters]
 
 
-class OperationFill(BaseModel):
+class OperationFill(BaseOperation):
     id: constant("fill")
     parameters: Optional[FillParameters]
 
@@ -103,7 +130,7 @@ class OperationFill(BaseModel):
         extra = "forbid"
 
 
-class OperationFilterResults(BaseModel):
+class OperationFilterResults(BaseOperation):
     id: constant("filter_results")
     parameters: Optional[Any]
 
@@ -118,7 +145,7 @@ class FilterResultsTopNParameters(BaseModel):
         extra = "forbid"
 
 
-class OperationFilterResultsTopN(BaseModel):
+class OperationFilterResultsTopN(BaseOperation):
     id: constant("filter_results_top_n")
     parameters: Optional[FilterResultsTopNParameters]
 
@@ -126,7 +153,7 @@ class OperationFilterResultsTopN(BaseModel):
         extra = "forbid"
 
 
-class OperationFilterKgraph(BaseModel):
+class OperationFilterKgraph(BaseOperation):
     id: constant("filter_kgraph")
     parameters: Optional[Any]
 
@@ -149,7 +176,7 @@ class FilterKgraphContinuousKedgeAttributeParameters(BaseModel):
     qnode_keys: HashableSequence[str] = []
 
 
-class OperationFilterKgraphContinuousKedgeAttribute(BaseModel):
+class OperationFilterKgraphContinuousKedgeAttribute(BaseOperation):
     id: constant("filter_kgraph_continuous_kedge_attribute")
     parameters: FilterKgraphContinuousKedgeAttributeParameters
 
@@ -164,7 +191,7 @@ class FilterKgraphDiscreteKedgeAttributeParameters(BaseModel):
     qnode_keys: HashableSequence[str] = []
 
 
-class OperationFilterKgraphDiscreteKedgeAttribute(BaseModel):
+class OperationFilterKgraphDiscreteKedgeAttribute(BaseOperation):
     id: constant("filter_kgraph_discrete_kedge_attribute")
     parameters: FilterKgraphDiscreteKedgeAttributeParameters
 
@@ -178,7 +205,7 @@ class FilterKgraphDiscreteKnodeAttributeParameters(BaseModel):
     qnode_keys: Optional[HashableSequence[str]]
 
 
-class OperationFilterKgraphDiscreteKnodeAttribute(BaseModel):
+class OperationFilterKgraphDiscreteKnodeAttribute(BaseOperation):
     id: constant("filter_kgraph_discrete_knode_attribute")
     parameters: FilterKgraphDiscreteKnodeAttributeParameters
 
@@ -186,7 +213,7 @@ class OperationFilterKgraphDiscreteKnodeAttribute(BaseModel):
         extra = "forbid"
 
 
-class OperationFilterKgraphOrphans(BaseModel):
+class OperationFilterKgraphOrphans(BaseOperation):
     id: constant("filter_kgraph_orphans")
     parameters: Optional[Any]
 
@@ -217,7 +244,7 @@ class FilterKgraphPercentileParameters(BaseModel):
     qnode_keys: HashableSequence[str] = []
 
 
-class OperationFilterKgraphPercentile(BaseModel):
+class OperationFilterKgraphPercentile(BaseOperation):
     id: constant("filter_kgraph_percentile")
     parameters: FilterKgraphPercentileParameters
 
@@ -241,7 +268,7 @@ class FilterKgraphStdDevParameters(BaseModel):
     qnode_keys: HashableSequence[str] = []
 
 
-class OperationFilterKgraphStdDev(BaseModel):
+class OperationFilterKgraphStdDev(BaseOperation):
     id: constant("filter_kgraph_std_dev")
     parameters: FilterKgraphStdDevParameters
 
@@ -249,7 +276,7 @@ class OperationFilterKgraphStdDev(BaseModel):
         extra = "forbid"
 
 
-class OperationFilterKgraphTopN(BaseModel):
+class OperationFilterKgraphTopN(BaseOperation):
     id: constant("filter_kgraph_top_n")
     parameters: FilterKgraphTopNParameters
 
@@ -257,7 +284,7 @@ class OperationFilterKgraphTopN(BaseModel):
         extra = "forbid"
 
 
-class OperationLookup(BaseModel):
+class OperationLookup(BaseOperation):
     id: constant("lookup")
     parameters: Optional[Any]
 
@@ -265,7 +292,7 @@ class OperationLookup(BaseModel):
         extra = "forbid"
 
 
-class OperationOverlay(BaseModel):
+class OperationOverlay(BaseOperation):
     id: constant("overlay")
     parameters: Optional[Any]
 
@@ -279,7 +306,7 @@ class OverlayComputeJaccardParameters(BaseModel):
     virtual_relation_label: str
 
 
-class OperationOverlayComputeJaccard(BaseModel):
+class OperationOverlayComputeJaccard(BaseOperation):
     id: constant("overlay_compute_jaccard")
     parameters: OverlayComputeJaccardParameters
 
@@ -292,7 +319,7 @@ class OverlayComputeNgdParameters(BaseModel):
     virtual_relation_label: str
 
 
-class OperationOverlayComputeNgd(BaseModel):
+class OperationOverlayComputeNgd(BaseOperation):
     id: constant("overlay_compute_ngd")
     parameters: OverlayComputeNgdParameters
 
@@ -300,7 +327,7 @@ class OperationOverlayComputeNgd(BaseModel):
         extra = "forbid"
 
 
-class OperationOverlayConnectKnodes(BaseModel):
+class OperationOverlayConnectKnodes(BaseOperation):
     id: constant("overlay_connect_knodes")
     parameters: Optional[Any]
 
@@ -315,7 +342,7 @@ class OverlayFisherExactTestParameters(BaseModel):
     rel_edge_key: Optional[str]
 
 
-class OperationOverlayFisherExactTest(BaseModel):
+class OperationOverlayFisherExactTest(BaseOperation):
     id: constant("overlay_fisher_exact_test")
     parameters: OverlayFisherExactTestParameters
 
@@ -323,7 +350,7 @@ class OperationOverlayFisherExactTest(BaseModel):
         extra = "forbid"
 
 
-class OperationRestate(BaseModel):
+class OperationRestate(BaseOperation):
     id: constant("restate")
     parameters: Optional[Any]
 
@@ -331,7 +358,7 @@ class OperationRestate(BaseModel):
         extra = "forbid"
 
 
-class OperationScore(BaseModel):
+class OperationScore(BaseOperation):
     id: constant("score")
     parameters: Optional[Any]
 
@@ -339,7 +366,7 @@ class OperationScore(BaseModel):
         extra = "forbid"
 
 
-class OperationSortResults(BaseModel):
+class OperationSortResults(BaseOperation):
     id: constant("sort_results")
     parameters: Optional[Any]
 
@@ -360,7 +387,7 @@ class SortResultsEdgeAttributeParameters(BaseModel):
     qedge_keys: Optional[HashableSequence[str]]
 
 
-class OperationSortResultsEdgeAttribute(BaseModel):
+class OperationSortResultsEdgeAttribute(BaseOperation):
     id: constant("sort_results_edge_attribute")
     parameters: SortResultsEdgeAttributeParameters
 
@@ -374,7 +401,7 @@ class SortResultsNodeAttributeParameters(BaseModel):
     qnode_keys: Optional[HashableSequence[str]]
 
 
-class OperationSortResultsNodeAttribute(BaseModel):
+class OperationSortResultsNodeAttribute(BaseOperation):
     id: constant("sort_results_node_attribute")
     parameters: SortResultsNodeAttributeParameters
 
@@ -386,7 +413,7 @@ class SortResultsScoreParameters(BaseModel):
     ascending_or_descending: AscOrDescEnum
 
 
-class OperationSortResultsScore(BaseModel):
+class OperationSortResultsScore(BaseOperation):
     id: constant("sort_results_score")
     parameters: SortResultsScoreParameters
 
